<?xml version="1.0" encoding="UTF-8"?>
<!-- ILCD Format Version 1.1_SNAPSHOT Tools Build 1016 -->
<xsl:stylesheet version="1.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform" xmlns:exslt="http://exslt.org/common" xmlns:categories="http://lca.jrc.it/ILCD/Categories"
    xmlns:locations="http://lca.jrc.it/ILCD/Locations" xmlns:lciamethodologies="http://lca.jrc.it/ILCD/LCIAMethodologies" xmlns:process="http://lca.jrc.it/ILCD/Process"
    xmlns:lciamethod="http://lca.jrc.it/ILCD/LCIAMethod" xmlns:flow="http://lca.jrc.it/ILCD/Flow" xmlns:flowproperty="http://lca.jrc.it/ILCD/FlowProperty"
    xmlns:unitgroup="http://lca.jrc.it/ILCD/UnitGroup" xmlns:source="http://lca.jrc.it/ILCD/Source" xmlns:contact="http://lca.jrc.it/ILCD/Contact"
    xmlns:common="http://lca.jrc.it/ILCD/Common" xmlns:epd="http://www.iai.kit.edu/EPD/2013" xmlns:epd2="http://www.indata.network/EPD/2019" xmlns:matml="http://www.matml.org/">

    <xsl:import href="validate-common-1.0.xsl"/>
    <xsl:import href="validate-references-1.0.xsl"/>

    <xsl:output indent="no" method="text"/>

    <xsl:variable name="version" select="'1.1'"/>

    <xsl:variable name="allowedModules" select="'|A1|A2|A3|A4|A5|A1-A3|B1|B2|B3|B4|B5|B6|B7|C1|C2|C3|C4|D|'"/>

    <xsl:variable name="fancyMessages" select="false()"/>

    <xsl:template match="/">
        <xsl:apply-templates/>
    </xsl:template>


    <xsl:template match="process:processDataSet">
        <xsl:call-template name="checkEPD12Version"/>
        <xsl:call-template name="checkFieldsPresence"/>
        <xsl:call-template name="checkModules"/>
        <xsl:call-template name="checkBackgroundData"/>
    </xsl:template>

    <xsl:template name="checkEPD12Version">
        <xsl:call-template name="checkPresence">
            <xsl:with-param name="element" select="/process:processDataSet[@epd2:epd-version='1.2']"/>
            <xsl:with-param name="elementDesc">On root element, attribute @epd-version with value "1.2"</xsl:with-param>
            <xsl:with-param name="suppressDefaultPrefix" select="'true'"/>
        </xsl:call-template>
    </xsl:template>

    <xsl:template match="flow:flowDataSet">
    	<xsl:variable name="uuid" select="/flow:flowDataSet/flow:flowInformation/flow:dataSetInformation/flow:UUID/@text"/>
    	<!--  exclude the LCI indicator flows -->
    	<xsl:if test="$uuid!='20f32be5-0398-4288-9b6d-accddd195317' or 
    				  $uuid!='fb3ec0de-548d-4508-aea5-00b73bf6f702' or 
    				  $uuid!='53f97275-fa8a-4cdd-9024-65936002acd0' or 
    				  $uuid!='ac857178-2b45-46ec-892a-a9a4332f0372' or 
    				  $uuid!='1421caa0-679d-4bf4-b282-0eb850ccae27' or 
    				  $uuid!='06159210-646b-4c8d-8583-da9b3b95a6c1' or 
    				  $uuid!='c6a1f35f-2d09-4f54-8dfb-97e502e1ce92' or 
    				  $uuid!='64333088-a55f-4aa2-9a31-c10b07816787' or 
    				  $uuid!='89def144-d39a-4287-b86f-efde453ddcb2' or 
    				  $uuid!='3cf952c8-f3a4-461d-8c96-96456ca62246' or 
    				  $uuid!='430f9e0f-59b2-46a0-8e0d-55e0e84948fc' or 
    				  $uuid!='b29ef66b-e286-4afa-949f-62f1a7b4d7fa' or 
    				  $uuid!='3449546e-52ad-4b39-b809-9fb77cea8ff6' or 
    				  $uuid!='a2b32f97-3fc7-4af2-b209-525bc6426f33' or 
    				  $uuid!='d7fe48a5-4103-49c8-9aae-b0b5dfdbd6ae' or 
    				  $uuid!='59a9181c-3aaf-46ee-8b13-2b3723b6e447' or 
    				  $uuid!='4da0c987-2b76-40d6-9e9e-82a017aaaf29' or 
    				  $uuid!='98daf38a-7a79-46d3-9a37-2b7bd0955810'">
	        <xsl:call-template name="checkFlowReferenceFlowPropertyPresence"/>
    	</xsl:if>
    </xsl:template>
    
    <xsl:template name="checkFlowReferenceFlowPropertyPresence">
        
        <xsl:variable name="numberOfDeclaredFlowProperties" select="count(/flow:flowDataSet/flow:flowProperties/flow:flowProperty)"/>
        
        <xsl:choose>
            <xsl:when test="$numberOfDeclaredFlowProperties=0">
                <xsl:call-template name="validationEvent">
                    <xsl:with-param name="message">No flow properties are declared for the reference flow.</xsl:with-param>
                </xsl:call-template>                              
            </xsl:when>
            <xsl:otherwise>
                <xsl:if test="count(/flow:flowDataSet/flow:flowProperties/flow:flowProperty[@dataSetInternalID=/flow:flowDataSet/flow:flowInformation/flow:quantitativeReference/flow:referenceToReferenceFlowProperty])=0">
                    <xsl:call-template name="validationEvent">
                        <xsl:with-param name="message">No reference flow property is declared for the reference flow.</xsl:with-param>
                    </xsl:call-template>                                                  
                </xsl:if>
            </xsl:otherwise>
        </xsl:choose>
    </xsl:template>
    
    <xsl:template name="checkModules">
        <xsl:for-each select="//common:other/epd:amount">
            <xsl:variable name="module" select="concat('|', @epd:module, '|')"/>
            <!-- check module name -->
            <xsl:choose>
                <xsl:when test="$module='||'">
                    <xsl:call-template name="validationEvent">
                        <xsl:with-param name="message">No module declared for value <xsl:value-of select="./text()"/></xsl:with-param>
                    </xsl:call-template>
                </xsl:when>
                <xsl:otherwise>
                    <xsl:if test="not(contains($allowedModules, $module))">
                        <xsl:call-template name="validationEvent">
                            <xsl:with-param name="message">Module name '<xsl:value-of select="@epd:module"/>' is invalid. Only one of <xsl:value-of select="$allowedModules"/> is allowed.</xsl:with-param>                            
                        </xsl:call-template>
                    </xsl:if>
                </xsl:otherwise>
            </xsl:choose>            
        </xsl:for-each>
        
        <!-- if D is declared, also C must be declared -->
        <xsl:if test="count(//common:other//epd:amount[@epd:module='D' and count(preceding-sibling::epd:amount[starts-with(@epd:module,'C')])=0 and count(following-sibling::epd:amount[starts-with(@epd:module,'C')])=0])!=0">
            <xsl:call-template name="validationEvent">
                <xsl:with-param name="message">A value for module D has been declared for at least one indicator without declaring a value for module C, which is required in this case.</xsl:with-param>                            
            </xsl:call-template>
        </xsl:if>
    </xsl:template>
    
    <xsl:template name="checkBackgroundData">
        <xsl:if test="count(/process:processDataSet/process:modellingAndValidation/process:dataSourcesTreatmentAndRepresentativeness/process:referenceToDataSource[@refObjectId='28d74cc0-db8b-4d7e-bc44-5f6d56ce0c4a' or @refObjectId='b497a91f-e14b-4b69-8f28-f50eb1576766'])=0">
            <xsl:call-template name="validationEvent">
                <xsl:with-param name="message">No mandatory background database (GaBi, ecoinvent) has been declared under datasources.</xsl:with-param>                            
            </xsl:call-template>
        </xsl:if>
    </xsl:template>

    <xsl:template name="checkFieldsPresence">
        <xsl:call-template name="checkPresence">
            <xsl:with-param name="element" select="process:processInformation/process:dataSetInformation/common:UUID"/>
            <xsl:with-param name="elementDesc" select="'UUID'"/>
            <xsl:with-param name="messagePrefix"/>
        </xsl:call-template>

        <xsl:call-template name="checkPresence">
            <xsl:with-param name="element" select="process:processInformation/process:dataSetInformation/process:name/process:baseName"/>
            <xsl:with-param name="elementDesc" select="'name'"/>
            <xsl:with-param name="messagePrefix"/>
        </xsl:call-template>

       <xsl:call-template name="checkPresence">
            <xsl:with-param name="element" select="process:processInformation/process:quantitativeReference/process:referenceToReferenceFlow"/>
            <xsl:with-param name="elementDesc" select="'reference to reference flow in section quantitative reference'"/>
            <xsl:with-param name="messagePrefix"/>
        </xsl:call-template>

        <xsl:call-template name="checkPresence">
            <xsl:with-param name="element" select="process:processInformation/process:time/common:referenceYear"/>
            <xsl:with-param name="elementDesc" select="'reference year'"/>
            <xsl:with-param name="messagePrefix"/>
        </xsl:call-template>

        <xsl:call-template name="checkPresence">
            <xsl:with-param name="element" select="process:processInformation/process:time/common:dataSetValidUntil"/>
            <xsl:with-param name="elementDesc" select="'data set valid until'"/>
            <xsl:with-param name="messagePrefix"/>
        </xsl:call-template>

        <xsl:call-template name="checkPresence">
            <xsl:with-param name="element" select="process:processInformation/process:geography/process:locationOfOperationSupplyOrProduction/@location"/>
            <xsl:with-param name="elementDesc" select="'location'"/>
            <xsl:with-param name="messagePrefix"/>
        </xsl:call-template>

        <xsl:call-template name="checkValue">
            <xsl:with-param name="element" select="process:modellingAndValidation/process:LCIMethodAndAllocation/process:typeOfDataSet"/>
            <xsl:with-param name="value1" select="'EPD'"/>
            <xsl:with-param name="elementDesc" select="'type of dataset'"/>
        </xsl:call-template>

        <xsl:call-template name="checkPresence">
            <xsl:with-param name="element" select="process:modellingAndValidation/process:LCIMethodAndAllocation/common:other/epd:subType"/>
            <xsl:with-param name="elementDesc" select="'subtype of dataset'"/>
            <xsl:with-param name="messagePrefix"/>
        </xsl:call-template>

        <xsl:call-template name="checkValue">
            <xsl:with-param name="element" select="process:modellingAndValidation/process:LCIMethodAndAllocation/common:other/epd:subType"/>
            <xsl:with-param name="value1" select="'generic dataset'"/>
            <xsl:with-param name="value2" select="'representative dataset'"/>
            <xsl:with-param name="value3" select="'average dataset'"/>
            <xsl:with-param name="value4" select="'specific dataset'"/>
            <xsl:with-param name="value5" select="'template dataset'"/>
            <xsl:with-param name="elementDesc" select="'type of dataset'"/>
        </xsl:call-template>

        <!-- TODO: exactly one default scenario per group -->
        <!--        <xsl:if test="count(process:processInformation/process:dataSetInformation/common:other/epd:scenarios/epd:scenario)>0">
            <xsl:call-template name="checkPresence">
                <xsl:with-param name="element" select="process:processInformation/process:dataSetInformation/common:other/epd:scenarios/epd:scenarinno[@epd:default='true']"/>
                <xsl:with-param name="elementDesc" select="'Exactly one default scenario per group'"/>
                <xsl:with-param name="suppressDefaultPrefix" select="'true'"/>
                <xsl:with-param name="messagePrefix"/>
            </xsl:call-template>
        </xsl:if>
-->

        <xsl:call-template name="checkPresence">
            <xsl:with-param name="element" select="process:modellingAndValidation/process:validation/process:review/common:referenceToNameOfReviewerAndInstitution"/>
            <xsl:with-param name="elementDesc" select="'Reviewer name and institution'"/>
            <xsl:with-param name="suppressDefaultPrefix" select="'true'"/>
            <xsl:with-param name="messagePrefix"/>
        </xsl:call-template>

        <xsl:call-template name="checkPresence">
            <xsl:with-param name="element"
                select="process:administrativeInformation/process:dataEntryBy/common:referenceToDataSetFormat[@refObjectId = 'a97a0155-0234-4b87-b4ce-a45da52f2a40']"/>
            <xsl:with-param name="elementDesc" select="'Declaration of ILCD format 1.1'"/>
            <xsl:with-param name="suppressDefaultPrefix" select="'true'"/>
            <xsl:with-param name="messagePrefix"/>
        </xsl:call-template>

        <xsl:call-template name="checkPresence">
            <xsl:with-param name="element"
                select="process:administrativeInformation/process:dataEntryBy/common:referenceToDataSetFormat[@refObjectId = 'a29449fd-aa2f-4de8-b5d7-4b06b43c6fde']"/>
            <xsl:with-param name="elementDesc" select="'Declaration of EPD Data Format Extensions 1.2'"/>
            <xsl:with-param name="suppressDefaultPrefix" select="'true'"/>
            <xsl:with-param name="messagePrefix"/>
        </xsl:call-template>

        <xsl:call-template name="checkPresence">
            <xsl:with-param name="element" select="process:administrativeInformation/process:publicationAndOwnership/common:dataSetVersion"/>
            <xsl:with-param name="elementDesc" select="'dataset version'"/>
            <xsl:with-param name="messagePrefix"/>
        </xsl:call-template>

        <xsl:call-template name="checkPresence">
            <xsl:with-param name="element" select="process:administrativeInformation/process:publicationAndOwnership/common:referenceToOwnershipOfDataSet"/>
            <xsl:with-param name="elementDesc" select="'owner of dataset'"/>
            <xsl:with-param name="messagePrefix"/>
        </xsl:call-template>

        <xsl:apply-templates/>

    </xsl:template>

    <xsl:template match="*[local-name()='classificationInformation']" priority="1000"/>

</xsl:stylesheet>
