<?xml version="1.0" encoding="UTF-8"?>
<!-- stylesheet for references validation v1.0 -->
<xsl:stylesheet version="1.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform" xmlns:exslt="http://exslt.org/common" xmlns:categories="http://lca.jrc.it/ILCD/Categories"
    xmlns:locations="http://lca.jrc.it/ILCD/Locations" xmlns:lciamethodologies="http://lca.jrc.it/ILCD/LCIAMethodologies" xmlns:process="http://lca.jrc.it/ILCD/Process"
    xmlns:lciamethod="http://lca.jrc.it/ILCD/LCIAMethod" xmlns:flow="http://lca.jrc.it/ILCD/Flow" xmlns:flowproperty="http://lca.jrc.it/ILCD/FlowProperty"
    xmlns:unitgroup="http://lca.jrc.it/ILCD/UnitGroup" xmlns:source="http://lca.jrc.it/ILCD/Source" xmlns:contact="http://lca.jrc.it/ILCD/Contact"
    xmlns:common="http://lca.jrc.it/ILCD/Common">

    <xsl:import href="common-1.0.xsl"/>
    
    <xsl:variable name="referenceTypes" select="document('reference_types-1.0.xml')/referenceTypes"/>
    
    <!-- check references -->
    <xsl:template
        match="*[local-name()='referenceToCommissioner'          
        or local-name()='referenceToComplementingProcess' 
        or local-name()='referenceToCompleteReviewReport'             
        or local-name()='referenceToComplianceSystem' 
        or local-name()='referenceToContact'             
        or local-name()='referenceToConvertedOriginalDataSetFrom' 
        or local-name()='referenceToDataHandlingPrinciples' 
        or local-name()='referenceToDataSetFormat' 
        or local-name()='referenceToDataSetUseApproval' 
        or local-name()='referenceToDataSource' 
        or local-name()='referenceToEntitiesWithExclusiveAccess' 
        or local-name()='referenceToExternalDocumentation'             
        or local-name()='referenceToFlowDataSet' 
        or local-name()='referenceToFlowPropertyDataSet' 
        or local-name()='referenceToIncludedProcesses' 
        or local-name()='referenceToIncludedSubMethods' 
        or local-name()='referenceToLCAMethodDetails' 
        or local-name()='referenceToLCIAMethodDataSet' 
        or local-name()='referenceToLogo' 
        or local-name()='referenceToNameOfReviewerAndInstitution' 
        or local-name()='referenceToOriginalEPD'
        or local-name()='referenceToOwnershipOfDataSet' 
        or local-name()='referenceToPersonOrEntityEnteringTheData' 
        or local-name()='referenceToPersonOrEntityGeneratingTheDataSet' 
        or local-name()='referenceToPrecedingDataSetVersion' 
        or local-name()='referenceToPrecedingDataSetVersion' 
        or local-name()='referenceToPublisher' 
        or local-name()='referenceToReferenceUnitGroup' 
        or local-name()='referenceToRegistrationAuthority' 
        or local-name()='referenceToSource' 
        or local-name()='referenceToSupportedImpactAssessmentMethods' 
        or local-name()='referenceToTechnicalSpecification' 
        or local-name()='referenceToTechnologyFlowDiagrammOrPicture' 
        or local-name()='referenceToTechnologyPictogramme' 
        or local-name()='referenceToUnchangedRepublication'
        or local-name()='referenceToVendor'
        or local-name()='isA']">
        
        <xsl:call-template name="checkReference"/>
    </xsl:template>
   
   
    <!-- check a reference -->
    <xsl:template name="checkReference">
        
        <xsl:variable name="elementName" select="name()"/>
        <xsl:variable name="typeFound" select="@type"/>
        
        <!-- check for correct type attribute  -->
        <xsl:variable name="correctType">
            <xsl:call-template name="determineReferenceType">
                <xsl:with-param name="for" select="local-name()"/>
                <xsl:with-param name="referenceTypes" select="$referenceTypes"/>
            </xsl:call-template>
        </xsl:variable>
        
        <xsl:if test="@type!=$correctType">
            <xsl:call-template name="validationEvent">
                <xsl:with-param name="fancyMessages" select="$fancyMessages"/>
                <xsl:with-param name="message">Element <xsl:value-of select="name()"/>: reference type "<xsl:value-of select="$correctType"/>" expected, but "<xsl:value-of
                    select="@type"/>" found.</xsl:with-param>
            </xsl:call-template>
        </xsl:if>
        
        <!-- reference must have either refObjectId or uri, or both -->
        <xsl:if test="not(@refObjectId or @uri)">
            <xsl:call-template name="validationEvent">
                <xsl:with-param name="fancyMessages" select="$fancyMessages"/>
                <xsl:with-param name="message">Element <xsl:value-of select="$elementName"/>: a reference must have either refObjectId or uri attribute, or both.</xsl:with-param>
            </xsl:call-template>
        </xsl:if>
        
        <!-- check if subReference element is allowed -->
        <xsl:if test="count(common:subReference)>0">
            <xsl:if
                test="$typeFound!='source data set' or local-name()='referenceToDataSetFormat' or local-name()='referenceToComplianceSystem' or local-name()='referenceToPrecedingDataSetVersion' or local-name()='referenceToTechnologyFlowDiagrammOrPicture' or local-name()='referenceToTechnologyPictogramme' or local-name()='logo'">
                <xsl:call-template name="validationEvent">
                    <xsl:with-param name="fancyMessages" select="$fancyMessages"/>
                    <xsl:with-param name="message">Element <xsl:value-of select="$elementName"/>: a subReference element is not allowed for this reference.</xsl:with-param>
                </xsl:call-template>
            </xsl:if>
        </xsl:if>
        
    </xsl:template>
    
</xsl:stylesheet>
